package drools.test;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.junit.Test;

import bitronix.tm.BitronixTransactionManager;
import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class PersistenceTest {

	boolean persistence = false;

	@Test
	public void testPersistence() throws Exception {

		PoolingDataSource ds = new PoolingDataSource();
		ds.setUniqueName("jdbc/processInstanceDS");
		ds.setClassName("org.h2.jdbcx.JdbcDataSource");
		ds.setMaxPoolSize(3);
		ds.setAllowLocalTransactions(true);
		ds.getDriverProperties().put("user", "sa");
		ds.getDriverProperties().put("password", "sasa");
		ds.getDriverProperties().put("URL", "jdbc:h2:file:./data/process-instance-db");
		ds.init();

		KnowledgeBuilder knowledgeBuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
		knowledgeBuilder.add(ResourceFactory.newClassPathResource("test.rf"), ResourceType.DRF);
		KnowledgeBase kbase = knowledgeBuilder.newKnowledgeBase();

		// create the entity manager factory and register it in the environment
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.drools.persistence.jpa");
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
		BitronixTransactionManager transactionManager = TransactionManagerServices.getTransactionManager();
		env.set(EnvironmentName.TRANSACTION_MANAGER, transactionManager);

		// create a new knowledge session that uses JPA to store the runtime
		// state
		StatefulKnowledgeSession ksession = null;
		if (persistence) {
			ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null, env);
		} else {
			ksession = kbase.newStatefulKnowledgeSession();
		}
		startProcesses(ksession, 10);
		transactionManager.shutdown();
		System.out.println("Test complete");
	}

	private List<Exception> failures = new ArrayList<Exception>();

	private void startProcesses(final StatefulKnowledgeSession ksession, int numProcesses) throws Exception {
		List<Thread> threads = new ArrayList<Thread>();
		for (int i = 0; i < numProcesses; i++) {
			Runnable r = new Runnable() {
				@Override
				public void run() {
					try {
						startProcess(ksession);
					} catch (Exception e) {
						failures.add(e);
					}
				}
			};
			Thread thread = new Thread(r, "ProcessStarterThread-" + i);
			thread.start();
			threads.add(thread);
		}
		for (Thread thread : threads) {
			thread.join();
		}
	}

	private void startProcess(StatefulKnowledgeSession ksession) {
		ksession.startProcess("TestWorkflow");
	}
}
